# ---------------------------------------------------------------------------
# clip_to_counties.py
# revision 03
# Model created on: Fri Feb 09 2007 04:11:43 PM
#   (generated by ArcGIS/ModelBuilder)
# Model revised on: Wed May 21 2008 042:00:00 PM
# Author: John Reiser <reiser@rowan.edu>
# Usage: ClipToCounties <inputs, ;-delimited> <output workspace>
# ---------------------------------------------------------------------------

#
# Remember to change the location of the County Layer!
#

# Import system modules

import sys, string, os, arcgisscripting
gp = arcgisscripting.create()

counties = ('ATLANTIC', 'BERGEN', 'BURLINGTON', 'CAMDEN', 'CAPE MAY', 'CUMBERLAND', 'ESSEX', 'GLOUCESTER', 'HUDSON', 'HUNTERDON', 'MERCER', 'MIDDLESEX', 'MORRIS', 'MONMOUTH', 'OCEAN', 'PASSAIC', 'SALEM', 'SOMERSET', 'SUSSEX', 'UNION', 'WARREN')
county_shp = "" # feature class used for county clipping

# Load required toolboxes...
gp.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Data Management Tools.tbx")
gp.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Analysis Tools.tbx")

#Split the semi-colon (;) delimited input string (tables or feature classes) into a list
def SplitMultiInputs(multiInputs):
    try:
        #Remove the single quotes and parathesis around each input featureclass
        multiInputs = multiInputs.replace("(","").replace(")","").replace("'","")

        #split input tables by semicolon ";"
        return multiInputs.split(";")
    except:
        raise Exception, msgErrorSplittingInput

if not gp.Exists(county_shp):
    gp.AddError("County Feature Class does not exist. Modify script to point to County Feature Class.")
    print gp.GetMessage(-1)
    sys.exit(1)

try:
    if len(sys.argv) < 3:
        print "Usage: ClipToCounties [inputs] [output workspace]"
        raise Exception, "Insufficient number of parameters provided"

# Script arguments...
    inLayers = sys.argv[1]
    inLayers = SplitMultiInputs(inLayers)

    outSpace = sys.argv[2]
    gp.workspace = outSpace
    wd = gp.Describe(outSpace)
    
    Selection_type = "NEW_SELECTION"
    Counties_Layer = "Counties"
    gp.MakeFeatureLayer(county_shp, Counties_Layer)

    for inLayer in inLayers:      # iterates through inputs
        if gp.Exists(inLayer):
          for county in counties: # iterates through counties
            # Modify SQL statement as needed
            SQLExpression = '"COUNTY" = \'' + county + "'"
           
            newName = os.path.basename(inLayer)
            newName = str(newName.replace(".shp", ""))
            outName = newName + "_" + county.title()
            outName = str(outName.replace(" ", "_"))
            outLayer = outSpace + os.sep + outName

            if wd.WorkspaceType == "FileSystem": # append .shp if output is shapefile
                outLayer = outLayer + ".shp"

            # Clip features based on current selected county
            # skips Counties already assumed clipped
            if gp.Exists(outLayer) == 0: # Exists returns 1/0 instead of True/False
                gp.SelectLayerByAttribute_management(Counties_Layer, Selection_type, SQLExpression)
                gp.AddWarning("Clipping " + outLayer + "...")
                print gp.GetMessage(-1)
                gp.Clip_analysis(inLayer, Counties_Layer, outLayer, "")
            else:
                gp.AddWarning(outLayer + " skipped.")
                print gp.GetMessage(-1)

except Exception, ErrorDesc:
    # Except block if the tool could not run at all.
    #  For example, not all parameters are provided, or if the output path doesn't exist.
    gp.AddError(str(ErrorDesc))
    print str(ErrorDesc)
