# ---------------------------------------------------------------------------
# CompactnessRatio.py
# revision 02
# Created on: Thu Mar 08 2007 09:21:30 AM
#   (generated by ArcGIS/ModelBuilder)
# Revised on: Wed May 21 2008 04:00:00 PM
# Author: John Reiser <reiser@rowan.edu>
#   Geography Department, Rowan University
# Usage: CompactnessRatio <Feature_Class> 
# ---------------------------------------------------------------------------

# Import system modules
import sys, string, os, arcgisscripting

gp = arcgisscripting.create() # Create the Geoprocessor object
gp.AddToolbox("C:/Program Files/ArcGIS/ArcToolbox/Toolboxes/Data Management Tools.tbx")

def CheckFields(shp):
    fex = 0 # variable to see if field exists
    gp.AddWarning("Checking file for AREA and PERIMETER fields.")
    print gp.GetMessage(-1)
    if not gp.ListFields(shp, "AREA").Next():
        gp.AddField_management(shp, "AREA", "DOUBLE", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")
        gp.AddWarning("Area Field Added.")
        print gp.GetMessage(-1)
        fex = 1
    if not gp.ListFields(shp, "PERIMETER").Next():
        gp.AddField_management(shp, "PERIMETER", "DOUBLE", "", "", "", "", "NULLABLE", "NON_REQUIRED", "")
        gp.AddWarning("Perimeter Field Added.")
        print gp.GetMessage(-1)
        fex = 1
    if fex == 1:
        CalculateAreaAndPerimeter(shp)

def CalculateAreaAndPerimeter(shp):
    calcs = {"AREA": "AREA", "LENGTH": "PERIMETER"}
    for key in calcs.keys():
        gp.AddWarning("Calculating", calcs[key].title())
        print gp.GetMessage(-1)
        gp.CalculateField_management(shp, calcs[key], "".join(["float(!SHAPE.", key, "!)"]), "PYTHON")

# Script arguments...
Feature_Class = sys.argv[1] # Makes any modifications in place
Output_Layer = "CalculateRatioLayer"

gp.MakeFeatureLayer_management(Feature_Class, Output_Layer, "", "", "")
desc = gp.Describe(Output_Layer)
type = desc.ShapeType
if type != "Polygon":
    gp.AddError("Input is not a polygon.")
    print gp.GetMessage(-1)
else:
    if(os.path.splitext(Feature_Class)[1] == ".shp"): #checks for shapefile
        gp.AddWarning("Feature Class is a shapefile.")
        print gp.GetMessage(-1)
        CheckFields(Feature_Class)
        CalculateAreaAndPerimeter(Feature_Class)
        Field_Name = "COMPACT" # Change CR field for shps here
        Code_Block = "pi = Atn(1) * 4\\ncr = (4 * pi * [AREA])/( [PERIMETER] * [PERIMETER])\\n"
    else:
        Field_Name = "COMPACTNESS" # Change CR field other FC types here
        Code_Block = "pi = Atn(1) * 4\\ncr = (4 * pi * [Shape_Area])/( [Shape_Length] * [Shape_Length])\\n"
    Field_Type = "DOUBLE"
    Field_Alias = "Compactness Ratio" # Change field alias here
    Expression = "cr"
    Expression_Type = "VB"
    Field_Precision = "7"
    Field_Scale = "6"

    if not gp.ListFields(Output_Layer, Field_Name).Next():
        gp.AddField_management(Feature_Class, Field_Name, Field_Type, Field_Precision, Field_Scale, "", Field_Alias, "NULLABLE", "NON_REQUIRED", "")
        gp.AddWarning("Adding Field...")
        print gp.GetMessage(-1)
    # Process: Calculate Field...
    gp.CalculateField_management(Feature_Class, Field_Name, Expression, Expression_Type, Code_Block)
    gp.AddMessage("Compactness ratio successfully calculated.")
    print gp.GetMessage(-1)
